package org.apache.bcel.classfile;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.DataInput;
import java.io.IOException;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ClassFormatException;
import org.apache.bcel.classfile.Code;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.ConstantUtf8;
import org.apache.bcel.classfile.ExceptionTable;
import org.apache.bcel.classfile.FieldOrMethod;
import org.apache.bcel.classfile.LineNumberTable;
import org.apache.bcel.classfile.LocalVariableTable;
import org.apache.bcel.classfile.ParameterAnnotationEntry;
import org.apache.bcel.classfile.Utility;
import org.apache.bcel.classfile.Visitor;
import org.apache.bcel.generic.Type;
import org.apache.bcel.util.BCELComparator;

public final class Method extends FieldOrMethod {
  public static final org.apache.bcel.classfile.Method[] EMPTY_ARRAY = new org.apache.bcel.classfile.Method[0];
  
  private static BCELComparator bcelComparator = (BCELComparator)new Object();
  
  static final org.apache.bcel.classfile.Method[] EMPTY_METHOD_ARRAY = new org.apache.bcel.classfile.Method[0];
  
  private ParameterAnnotationEntry[] parameterAnnotationEntries;
  
  public static BCELComparator getComparator() {
    return bcelComparator;
  }
  
  public static void setComparator(BCELComparator comparator) {
    bcelComparator = comparator;
  }
  
  public Method() {}
  
  Method(DataInput file, ConstantPool constantPool) throws IOException, ClassFormatException {
    super(file, constantPool);
  }
  
  public Method(int accessFlags, int nameIndex, int signatureIndex, Attribute[] attributes, ConstantPool constantPool) {
    super(accessFlags, nameIndex, signatureIndex, attributes, constantPool);
  }
  
  public Method(org.apache.bcel.classfile.Method c) {
    super(c);
  }
  
  public void accept(Visitor v) {
    v.visitMethod(this);
  }
  
  public org.apache.bcel.classfile.Method copy(ConstantPool constantPool) {
    return (org.apache.bcel.classfile.Method)copy_(constantPool);
  }
  
  public boolean equals(Object obj) {
    return bcelComparator.equals(this, obj);
  }
  
  public Type[] getArgumentTypes() {
    return Type.getArgumentTypes(getSignature());
  }
  
  public Code getCode() {
    for (Attribute attribute : getAttributes()) {
      if (attribute instanceof Code)
        return (Code)attribute; 
    } 
    return null;
  }
  
  public ExceptionTable getExceptionTable() {
    for (Attribute attribute : getAttributes()) {
      if (attribute instanceof ExceptionTable)
        return (ExceptionTable)attribute; 
    } 
    return null;
  }
  
  public LineNumberTable getLineNumberTable() {
    Code code = getCode();
    if (code == null)
      return null; 
    return code.getLineNumberTable();
  }
  
  public LocalVariableTable getLocalVariableTable() {
    Code code = getCode();
    if (code == null)
      return null; 
    return code.getLocalVariableTable();
  }
  
  public ParameterAnnotationEntry[] getParameterAnnotationEntries() {
    if (this.parameterAnnotationEntries == null)
      this.parameterAnnotationEntries = ParameterAnnotationEntry.createParameterAnnotationEntries(getAttributes()); 
    return this.parameterAnnotationEntries;
  }
  
  public Type getReturnType() {
    return Type.getReturnType(getSignature());
  }
  
  public int hashCode() {
    return bcelComparator.hashCode(this);
  }
  
  public String toString() {
    String access = Utility.accessToString(getAccessFlags());
    ConstantUtf8 c = getConstantPool().getConstantUtf8(getSignatureIndex());
    String signature = c.getBytes();
    c = getConstantPool().getConstantUtf8(getNameIndex());
    String name = c.getBytes();
    signature = Utility.methodSignatureToString(signature, name, access, true, getLocalVariableTable());
    StringBuilder buf = new StringBuilder(signature);
    for (Attribute attribute : getAttributes()) {
      if (!(attribute instanceof Code) && !(attribute instanceof ExceptionTable))
        buf.append(" [").append(attribute).append("]"); 
    } 
    ExceptionTable e = getExceptionTable();
    if (e != null) {
      String str = e.toString();
      if (!str.isEmpty())
        buf.append("\n\t\tthrows ").append(str); 
    } 
    return buf.toString();
  }
}
